<?php
/*------------------------------------------------------------------------
# scancircle.php - ScanCircle plugin for Joomla
# ------------------------------------------------------------------------
# author    aklaren
# copyright Copyright (C) 2014-2020 ScanCircle.com. All Rights Reserved.
# @license - http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
# Websites: https://www.scancircle.com
# Technical Support:  Forum - https://www.scancircle.com/forum
-------------------------------------------------------------------------*/
	defined('_JEXEC') or die('Restricted access');
	class plgContentScanCircle extends JPlugin {
		const SCANCIRCLE = '/\[scancircle .*?\]/i';
		const RESULTS = '/\[scancircle_results .*?\]/i';
		const PARAMS = '/\s+(\w+)="(.*?)"/';
		const PHPVAR = '/\[phpvar.*?\]/i';
		
		function onContentPrepare($context, &$article, &$params, $page = 0) {
			$found = false;
			while(preg_match(self::SCANCIRCLE, $article->text, $shortcode)) {
				if(!$found) {
					// *** IMPORTANT: always keep this version number in sync with the one in the XML ***
					$version = '2.4.1';
					$options = '?plugin=Joomla'.$version;
					$partner = '';
					$language = 'en';
					$secure = 's';
					$env = '';
					if(preg_match_all(self::PARAMS, $shortcode[0], $matches, PREG_SET_ORDER)) {
						foreach($matches as $param) {
							$key = $param[1];
							$val = $param[2];
							switch($key) {
								case 'partner':
									$partner = $val;
									break;
								case 'language':
									$language = $val;
									break;
								case 'https':
									if($val === '0') $secure = '';
									break;
								case 'env':
									$env = $val;
									break;
								default:
									$options .= '&'.$key.'='.rawurlencode($val);
							}
						}
						if($partner) {
							$found = true;
							$expanded = <<< EOL
<div id="scancircle">
	<a href="http://$partner.scancircle.com$env/$language/scan" id="scancircle_button" title="ScanCircle Scan">ScanCircle</a>
</div>
EOL;
							// Joomla strips <script> tags from content, so add to header
							$script = "http$secure://$partner.scancircle.com$env/$language/scancircle.js$options";
							if(strpos($options, '&speedometer=') !== false) {
								JHtml::_('jquery.framework');	// load jQuery if Speedometer is to be used
							}
							$document = JFactory::getDocument();
							$document->addScript($script);
						} else {
							$expanded = "*** Syntax: [scancircle partner=x [options]]: {$shortcode[0]} ***";
						}
					} else {
						$expanded = "*** ERROR: scancircle shortcode without parameters: {$shortcode[0]} ***";
					}
				} else {
					$expanded = '*** ERROR: only one scan widget allowed per page ***';
				}
				$article->text = preg_replace(self::SCANCIRCLE, $expanded, $article->text, 1);
			}

			while(preg_match(self::RESULTS, $article->text, $shortcode)) {
				$partner = '';
				$parameter = '';
				$attribs = '';
				if(preg_match_all(self::PARAMS, $shortcode[0], $matches, PREG_SET_ORDER)) {
					foreach($matches as $param) {
						$key = $param[1];
						$val = $param[2];
						switch($key) {
							case 'partner':
								$partner = $val;
								break;
							case 'param':
								$parameter = $val;
								break;
							default:
								if(strtolower($key) != 'src') $attribs .= ' '.$key.'='. htmlentities($val);
						}
					}
				}
				if(!$partner || !$parameter) {
					$expanded = 'Syntax: [scancircle_results partner=PARTNER-CODE param=URL-PARAMETER [IFRAME-ATTRIBUTES]]';
				} elseif(!preg_match('/^[a-z0-9]+$/i', $partner)) {
					$expanded = 'Invalid partner code: '.$partner;
				} elseif(!isset($_GET[$parameter])) {
					$expanded = 'Missing URL-parameter: '.$parameter;
				} elseif(!filter_var($url = $_GET[$parameter], FILTER_VALIDATE_URL)) {
					$expanded = 'Invalid URL: '.htmlentities($url);
				} elseif(!preg_match('/^https?:\/\/'.$partner.'\.scancircle\.com\//i', $url)) {
					$expanded = "Only valid for ScanCircle URL's for partner: ".$partner;
				} else {
					$expanded = '<iframe src="'.$url.'"'.$attribs.'></iframe>';
				}
				$article->text = preg_replace(self::RESULTS, $expanded, $article->text, 1);
			}

			while(preg_match(self::PHPVAR, $article->text, $matches)) {
				$shortcode = $matches[0];
				if($get = $this->getParameter($shortcode, 'get')) {
					$expanded = $_GET[$get];
				} elseif($post = $this->getParameter($shortcode, 'post')) {
					$expanded = $_POST[$post];
				} elseif($server = $this->getParameter($shortcode, 'server')) {
					$expanded = $_SERVER[$server];
				} else {
					$expanded = 'Syntax: [phpvar get|post|server="var"]';
				}
				
				$article->text = preg_replace(self::PHPVAR, $expanded, $article->text, 1);
			}
			
			return true;
		}
		
		function getParameter($shortcode, $param, $default=null) {
			// match: parameter name, equals, single/double quote, any (escaped) character or not being opening quote, opening quote
			return (preg_match('/\b'.$param.'=(["\'])((?:\\.|(?!\1).)*)\1/i', $shortcode, $matches) ?  $matches[2] :  $default);
		}
	}
